<?php

namespace Teknisa\Libs\Util;

use Zeedhi\Framework\DTO;
use Zeedhi\Framework\DataSource\DataSet;
use Teknisa\Libs\Util\Messages;

class ReportDecorator {
    
    const BIRT_PATH_PARAMETER = 'birtReportPath'; 
    const QR2_PATH_PARAMETER = 'qr2ReportPath';
    
    protected $container;
    protected $nrorg;
    protected $windowPrefix;
    
    public function __construct($environment) {
        $this->container = \Zeedhi\Framework\DependencyInjection\InstanceManager::getInstance()->getContainer();
        $this->nrorg = $environment->getNrorgTrab() != null ? $environment->getNrorgTrab(): $environment->getNrorg();
    }

    public function getData(DTO\Request\Row $request, DTO\Response $response) {
        try {
            $row = $request->getRow();
            $this->windowPrefix = $row[0];
            
            $reportsList = array();
            $reportsList = array_merge($this->getQr2Reports(), $reportsList);
            $reportsList = array_merge($this->getBirtReports(), $reportsList);
            
            $reportData = $this->buildData($reportsList);
            
            $response->setDataSets(array(new DataSet('dataReport', $reportData)));
            
        } catch (\Exception $e) {
            $response->setError(new DTO\Response\Error($e->getMessage(), 500));
        }
    }
    
    private function getQr2Reports() {
        return $this->getReportsMatched(self::QR2_PATH_PARAMETER, Messages::ERR_GET_QR2PATH_REPORTS_TO_BUILD_LIST);
    }
    
    private function getBirtReports() {
        return $this->getReportsMatched(self::BIRT_PATH_PARAMETER, Messages::ERR_GET_BIRTPATH_REPORTS_TO_BUILD_LIST);
    }

    private function buildData($reportList) {
        return array_map(function($report) {
            $report = explode('_', $report);
            $reportValue = $report[0];
            $reportDescription = str_replace('-', ' ', explode('.', $report[1])[0]);
            
            return array(
                'value' => $reportValue,
                'description' => $reportDescription
            );    
        }, $reportList);
    }

    private function getReportsMatched($parameterToFind, $errorMess) {
        if ($this->container->hasParameter($parameterToFind) && $this->container->getParameter($parameterToFind) != null) {
            $dirToScan = $this->container->getParameter($parameterToFind);
            if (is_dir($dirToScan)) {
                $reportList = scandir($dirToScan);
                if ($reportList != false) {
                    $dirNrorg = "$dirToScan/$this->nrorg";
                    if (is_dir($dirNrorg)) {
                        $reportListNrorg = scandir($dirNrorg);
                        $reportList = $reportListNrorg != false ? array_merge($reportList, $reportListNrorg) : $reportList;
                    }
                    return array_filter($reportList, function($report) {
                        return $this->matchReport($report);
                    });
                } else {
                    throw new \Exception($errorMess, 1);
                }
            } else {
                throw new \Exception("$dirToScan não é um diretório.", 1);
            }
        } else {
            return array();
        }
    }
    
    
    private function matchReport($report) {
        $regex = "/^(?i)$this->windowPrefix/";
        $resultMatch = preg_match($regex, $report);
        if ($resultMatch != 0) {
            return isset(explode("_", $report)[1]);
        } else {
            return false;
        }
    }
    
}