/* eslint-disable */
import {
  Metadata, IDictionary, IEventParam,
  Loader, Http, Router,
} from '@zeedhi/core';
import {
  Row, Form, IComponentRender, CheckboxMultiple,
  Button, Select, SelectMultiple, TextInput, Time, AlertService, Switch, Date as DateInput, DialogService,
} from '@zeedhi/common';
import { getRouteParams } from '@/router';

export class CadastroController {
  public modalController = Loader.getInstance('NovoCriterioController');
  public carregandoModal = Loader.getInstance('ModalLoading');

  public origem!: string;

  public setOrigem() {
    this.origem = (location.search.slice(1).split('=')[1]) ? '/Calendario' : '/Tarefas'
  }

  public dadosTarefa: IDictionary = {
    selectTipoLocal: [],
    selectFuncionario: [],
    selectTarefa: [],
    selectFuncionarioVistoria: [],
  };

  public setDados(nome: string, cod: number) {
    return {
      name: nome,
      value: cod,
    };
  }

  public setTipoLocal() {
    Http.get('/getTipoLocal').then((response: IDictionary) => {
      response.data.dataset.tipoLocal.forEach((dados: IDictionary) => {
        this.dadosTarefa.selectTipoLocal.push(this.setDados(dados.NMTIPOLOCALCLI, dados.NRTIPOLOCALCLI));
      });
    });
  }

  public setValuesLocalAndChecklist({ component }: { component:Select }) {
    this.setLocal(component.value);
    this.setTipoChecklist(component.value);
  }

  public setLocal(nr: number) {
    const selectLocal = Metadata.getInstance<Select>('selectNomeLocal');
    selectLocal.datasource.data = [];
    selectLocal.disabled = false;
    selectLocal.setValue([]);
    Http.get(`/getSpecificLocal?nr=${nr}`).then((response: IDictionary) => {
      response.data.dataset.Locals.forEach((dados: IDictionary) => {
        selectLocal.datasource.data.push({
          name: dados.NMLOCALCLIENTE,
          value: dados.NRLOCALCLIENTE,
          padrao: dados.NRCHECKLISTPADR,
        });
      });
    });
  }

  public setTipoChecklist(nr: number) {
    this.modifyFields(true);
    // Metadata.getInstance<Button>('BotaoNovoCriterio').disabled = true;
    const selectChecklist = Metadata.getInstance<SelectMultiple>('selectChecklist');
    selectChecklist.datasource.data = [];
    selectChecklist.disabled = false;
    selectChecklist.setValue([]);
    Http.get(`/getSpecificChecklist?nr=${nr}`).then((response: IDictionary) => {
      response.data.dataset.checklist.forEach((dados: IDictionary) => {
        selectChecklist.datasource.data.push(this.setDados(dados.NMCHECKLIST, dados.NRCHECKLIST));
      });
    });
  }

  public modifyFields(condicao: boolean, db: any[] = []) {
    Metadata.getInstance<Button>('BotaoNovoCriterio').disabled = condicao;
    const chkMult = Metadata.getInstance<CheckboxMultiple>('CheckBoxDeCriterios');
    chkMult!.datasource!.data = db;
    chkMult!.isVisible = !condicao;
    chkMult!.value = [];
  }

  public setDefaulValue({ component }: { component:Select }) {
    if (component.value !== null){
      const [dados] = component.datasource.data.filter((conjunto: IDictionary) => conjunto.value === component.value);
      const chkPadrao = Metadata.getInstance<SelectMultiple>('selectChecklist');
      const exist = chkPadrao.datasource.data.map((conjunto: IDictionary) => conjunto.value);
      if (chkPadrao.value === null && dados.padrao !== null && exist.includes(dados.padrao)){
        chkPadrao.setValue(dados.padrao);
      }
    }
  }

  public setItensChecklist({ component }: { component:Select }) {
    const selectItem = Metadata.getInstance<SelectMultiple>('selectTipoTarefa');
    const selectTarefa = Metadata.getInstance<SelectMultiple>('selectTarefa');

    this.modifyFields(true);
    const selectTipoTarefa: IDictionary = [];
    if (component.value !== null) {
      Http.post('/getItensChecklist', { checklist: component.value }).then((response: IDictionary) => {
        response.data.dataset.tipoTarefa.forEach((dados: IDictionary) => {
          selectTipoTarefa.push(this.setDados(dados.NMATIVIDADE, dados.NRATIVIDADECHK));
        });
      });
      selectItem.disabled = false;
    } else {
      selectItem.disabled = true;
      // Metadata.getInstance<Button>('BotaoNovoCriterio').disabled = true;
    }
    selectItem.datasource.data = selectTipoTarefa as IDictionary[];
    selectItem.setValue([]);
    selectTarefa.datasource.data = [];
    selectTarefa.setValue([]);
    selectTarefa.disabled = true;
  }

  public setTarefa({ component }: { component:Select }) {
    this.modifyFields(true);
    // Metadata.getInstance<Button>('BotaoNovoCriterio').disabled = true;
    const selectTipoTarefa: IDictionary = [];
    if (component.value !== null && component.value.length > 0) {
      Http.post('/getTarefa', { checklist: component.value }).then((response: IDictionary) => {
        response.data.dataset.tarefa.forEach((dados: IDictionary) => {
          selectTipoTarefa.push(this.setDados(dados.NMTAREFA, dados.NRTAREFACHK));
        });
      });
    };
    const select = Metadata.getInstance<SelectMultiple>('selectTarefa');
    select.datasource.data = selectTipoTarefa as IDictionary[];
    select.setValue([]);
    select.disabled = false;
  }

  public getCriteriosDeAvaliacao({ component }: { component:Select }) {
    if (component.value !== null && component.value.length) {
      this.modifyFields(false);
      Http.post('/getCriterioAtividade', {
        NRCHECKLIST: Metadata.getInstance<SelectMultiple>('selectChecklist').value,
        NRATIVIDADECHK: Metadata.getInstance<SelectMultiple>('selectTipoTarefa').value,
        NRTAREFACHK: Metadata.getInstance<SelectMultiple>('selectTarefa').value,
      }).then((response: IComponentRender) => {
        const dbChk: any[] = [];
        response.data.dataset.criterioTarefa.forEach((dados: IDictionary) => {
          dbChk.push({
            value: dados.DSCRITERIOTAREFA,
            id: dados.NRCRITERIOTAREFACHK,
          });
        });
        this.modifyFields(false,dbChk);
      });
    } else {
      this.modifyFields(true);
    }
  }

  public setFuncionario() {
    Http.get('/getFuncionario').then((response: IDictionary) => {
      response.data.dataset.funcionario.forEach((dados: IDictionary) => {
        this.dadosTarefa.selectFuncionario.push(this.setDados(dados.NMFUNCIONARIO, dados.NRFUNCIONARIO));
      });
    });
  }

  public setSupervisor() {
    Http.get('/getSupervisor').then((response: IDictionary) => {
      response.data.dataset.funcionario.forEach((dados: IDictionary) => {
        this.dadosTarefa.selectFuncionarioVistoria.push(this.setDados(dados.NMFUNCIONARIO, dados.NRFUNCIONARIO));
      });
    });
  }

  public inicializarDados() {
    this.setTipoLocal();
    this.setFuncionario();
    this.setSupervisor();
  }

  public getDadosInfo({ component }: IEventParam<Select>) {
    component.datasource.data = this.dadosTarefa[(component.name.includes('selectFuncionario')) ? 'selectFuncionario' : component.name];
  }

  public getDadosInfoSup({ component }: IEventParam<Select>) {
    component.datasource.data = this.dadosTarefa[(component.name.includes('selectFuncionarioVistoria')) ? 'selectFuncionarioVistoria' : component.name];
  }

  // Comandos do modal
  public createModal() {
    this.modalController.createBasicUsageModal();
  }

  public destroyModal() {
    this.modalController.destroyBasicUsageModal();
  }

  public openModal() {
    this.modalController.openBasicModal();
  }

  public fecharModal() {
    this.modalController.closeBasicModal();
  }

  public modelCrterio(criterio: IDictionary) {
    return {
      name: 'criterioTarefa',
      component: 'ZdCheckbox',
      label: criterio.name,
      cssClass: 'zd-mx-3',
      value: criterio.value,
    };
  }

  public novoCriterio() {
    const form = Metadata.getInstance<Form>('modalCriacaoCriterio');
    if (form.validate()) {
      const chk = Metadata.getInstance<CheckboxMultiple>('CheckBoxDeCriterios');
      const valor = Metadata.getInstance<TextInput>('InputNovoCriterio').value;
      chk!.datasource!.data!.push({
        value: valor,
        id: valor,
      });
      this.fecharModal();
    }
  }

  public botaoCriarTarefa() {
    /* eslint-disable no-console */
    const test = getRouteParams();
    // Variavel onde as datas irão ficar salva no formato de string
    const datas: IDictionary = [];
    // Pegando os valores de data do campo ZdDate
    let data = Metadata.getInstance<DateInput>('selectDataDe').value;
    let data2 = Metadata.getInstance<DateInput>('selectDataAte').value;
    // Criando uma validação se a data esta preenchida com todos os digitos
    // Verificando se a data final e igual ou superior a inicial
    let campoDataValido = ((data != null) ?  data.length === 10 : false) && ((data2 != null) ?  data2.length === 10 : false);
    if (campoDataValido) {
      // Transformando a variável em array para sebapara dia, mês e ano
      data = data.split('/');
      data2 = data2.split('/');
      // Usando as array da data para criar um Object Date.
      const inicio = Metadata.getInstance<Time>('selectHoraInicio').value;

      data = new Date(`${data[1]}/${data[0]}/${data[2]} ${inicio}`);
      data2 = new Date(`${data2[1]}/${data2[0]}/${data2[2]} ${inicio}`);
      campoDataValido = campoDataValido && (data <= data2);
    }
    // Validando todos os campos forms da tela e criando a variável com o estado da validação
    const form = Metadata.getInstance<Form>('FormularioPrincipal').validate();
    const form1 = Metadata.getInstance<Form>('FormComHorarioEMedia').validate();
    const form2 = Metadata.getInstance<Form>('FormComFuncionariosDaTarefa').validate();
    // Instanciando uma variavel com o campo do checkbox de datas
    const dias = Metadata.getInstance<CheckboxMultiple>('checkboxMultipleDate').value;
    if (!campoDataValido) {
      this.msg('Insira um periodo valida','info')
    } else if (dias.length === 0) {
      this.msg('Selecione o(s) dia(s) da semana','info');
    } else {
      const duracao = Metadata.getInstance<TextInput>('selectTempoMedio').value;
      // loop para percorrer as datas do período
      for (let dataAux = data; data <= data2; dataAux.setDate(dataAux.getDate() + 1)) {
        // Validação se o dia da data e compativel com os "Dias da semana" marcados
        if (dias.includes(dataAux.getDay())) {
          // Se a data iterada for compativel com o dia da semana ele vai salvar essa data em uma arry no formato string
          datas.push({ 
            DATAINICIAL: this.dataFormat(dataAux),
            DATAFINAL: this.dataFormat(new Date(dataAux.setMinutes(dataAux.getMinutes() + this.calculaMinuto(duracao)))),
          });
        }
      }
      /* Essa validação e para caso a pessoa tenha escolhido um periodo valido mas o dia escolhido não se encontra 
      nesse intervalo de datas */
      if (datas.length === 0) {
        // Exibe a msg com o erro
        this.msg('O periodo selecionado não possui o(s) dia(s) da semana escolhido.','warning');
      }
    }
    //Condicional para mandar os dados para o backend
    // Nessa condicional ele faz a validação de todos os forms estão preenchidos, e verifica se alguma data foi salva.
    if (form && form1 && form2 && datas.length > 0) {
      const selectObjectFuncionario = Metadata.getInstance<Select>('selectFuncionario').value;
      const selectObjectEncarregado = Metadata.getInstance<Select>('selectFuncionarioVistoria').value;
      const textoFuncionario =  'Você esta criando uma tarefa sem vincular um funcionário, deseja continuar?';
      const textoVistoria =  'Você esta criando uma tarefa sem vincular um encarregado, deseja continuar?';
      const semVinculo = 'Você esta criando uma tarefa sem vincular um funcionário e um encarregado, deseja continuar?'
      if (selectObjectFuncionario === null && selectObjectEncarregado === null){
        this.mensagem(semVinculo, datas);
      } else if(selectObjectFuncionario != null && selectObjectEncarregado === null) {
        this.mensagem(textoVistoria, datas);
        } else if (selectObjectFuncionario === null && selectObjectEncarregado != null) {
          this.mensagem(textoFuncionario, datas);
        }
        else {
        this.postTarefa(datas);
      }
    }
  }

  public criaNovoHorario(n: number) {
    return [
      {
        name: 'selectHoraInicio',
        component: 'ZdTime',
        autocomplete: false,
        cssClass: 'zd-px-2',
        label: 'Inicio',
        grid: {
          cols: 6,
        },
        validations: {
          required: {},
        },
      },
      {
        name: 'selectTempoMedio',
        component: 'ZdTextInput',
        cssClass: 'zd-px-2',
        label: 'Duração (minutos)',
        placeholder: '00:00',
        mask: '##:##',
        grid: {
          cols: 6,
        },
        validations: {
          required: {
            message: 'Campo Obrigatorio',
          },
          minLength: {
            limit: 5,
            message: 'Campo Obrigatorio',
          },
          maxLength: {
            limit: 5,
            message: 'Campo Obrigatorio',
          },
        },
      },
      {
        name: `selectFuncionario${n}`,
        component: 'ZdSelect',
        autocomplete: false,
        label: 'Funcionário',
        dataText: 'name',
        dataValue: 'value',
        validations: {
          required: {},
        },
        events: {
          onBeforeMount: '{{CadastroController.getDadosInfo}}',
          change: '{{CadastroController.validaCampoFuncionario}}',
        },
        datasource: {
          loadAll: true,
          data: [],
        },
        grid: {
          cols: 12,
        },
      },
    ];
  }

  public novoHorario() {
    Metadata.getInstance<Button>('RemoveNovoHorario').isVisible = true;
    const form = Metadata.getInstance<Form>('FormComHorarioEMedia');
    const row = Metadata.getInstance<Row>('FormComFuncionariosDaTarefa');
    form.children.splice(form.children.length - 1, 0, this.criaNovoHorario(0)[0] as IComponentRender);
    form.children.splice(form.children.length - 1, 0, this.criaNovoHorario(0)[1] as IComponentRender);
    row.children.push(this.criaNovoHorario(row.children.length)[2] as IComponentRender);
  }

  public removeHorario() {
    const form = Metadata.getInstance<Form>('FormComHorarioEMedia');
    const row = Metadata.getInstance<Row>('FormComFuncionariosDaTarefa');
    if (row.children.length > 1) {
      row.children.splice(row.children.length - 1, 1);
      form.children.splice(form.children.length - 3, 2);
      if (row.children.length === 1) {
        Metadata.getInstance<Button>('RemoveNovoHorario').isVisible = false;
        Metadata.getInstance<Switch>('switchFunionario').disabled = false;
      }
    }
  }

  public dataFormat(data: Date) {
        const dia  = data.getDate().toString();
        const diaF = (dia.length == 1) ? '0'+dia : dia;
        const mes  = (data.getMonth()+1).toString(); //+1 pois no getMonth Janeiro começa com zero.
        const mesF = (mes.length == 1) ? '0'+mes : mes;
        const anoF = data.getFullYear();
        const hora = data.getHours().toString();
        const horaF = (hora.length == 1) ? '0'+hora : hora;
        const minuto = data.getMinutes().toString();
        const minutoF = (minuto.length == 1) ? '0'+minuto : minuto;
    return anoF+"/"+mesF+"/"+diaF+" "+horaF+":"+minutoF;
  }

  public msg(text: string,tipo: string) {
    AlertService.show({
        name: 'basic-alert',
        text: text,
        top: true,
        left: false,
        timeout: 3000,
        color: tipo,
      });
  }

  public validaCampoFuncionario() {
    const dadosForm = Metadata.getInstance<Form>('FormComFuncionariosDaTarefa').value
    const validacao: IDictionary = []
    for (var property in dadosForm){
      if (dadosForm[property] != null) {
        (validacao.includes(dadosForm[property])) ? '' : validacao.push(dadosForm[property]);
      }
    }
    const swt = Metadata.getInstance<Switch>('switchFunionario');
    if (validacao.length > 1) {
      swt.value = false;
      swt.disabled = true;
    } else {
      swt.disabled = false;
    }
  }
  
  public mensagem (texto:  string, datas: IDictionary) {
  	DialogService.show({
  	  name: 'basic-dialog',
  	  type: 'warning',
  	  title: 'Atenção!',
  	  text: `${texto}`,
  	  buttons: [
      {
  		  name: 'cancel',
  		  component: 'ZdButton',
  		  outline: true,
  		  label: 'Voltar',
        events: { 
  		   click: () => DialogService.hide(),
  		  }
  		},
  		{
  		  name: 'yes',
  		  component: 'ZdButton',
  		  label: 'Criar',
  		  events: {
  		   click: () => this.postTarefa(datas),
  		  },
  		},
  	  ],
  	});
  }
  public postTarefa(datas: IDictionary) {
    DialogService.hide();
    this.carregandoModal.openPersistentModal();
    Http.post('/insertNovaTarefa', {
    	NMTAREFA: Metadata.getInstance<TextInput>('setNomeTarefa').value,
    	TIPOLOCAL: Metadata.getInstance<Select>('selectTipoLocal').value,
    	LOCAL: Metadata.getInstance<Select>('selectNomeLocal').value,
    	CHECKLIST: Metadata.getInstance<Select>('selectChecklist').value,
    	ITENSCHECK: Metadata.getInstance<Select>('selectTipoTarefa').value,
    	TAREFA: Metadata.getInstance<Select>('selectTarefa').value,
    	CRITERIOS: Metadata.getInstance<CheckboxMultiple>('CheckBoxDeCriterios').value,
    	DATA: datas,
    	NRFUNCIONARIO: Metadata.getInstance<Select>('selectFuncionario').value,
    	NRENCARREGADO: Metadata.getInstance<Select>('selectFuncionarioVistoria').value,
    }).then((response: IDictionary) => {
      const resposta = response.data.dataset.resposta.retorno;
      const erro = resposta
      const texto = `<p>Operação Bloqueada!</p><p> Ocorreram erros durante a inserção da tarefa:</p><div class= "erro">Código do Erro:${erro}</div>`;
    	if (resposta != 'OK'){
        this.carregandoModal.closePersistentModal();
        DialogService.show({
      	  name: 'basic-dialog',
      	  type: 'warning',
      	  title: 'Atenção!',
      	  text: `${texto}`, 
      	  buttons: [
          {
      		  name: 'cancel',
      		  component: 'ZdButton',
      		  outline: true,
      		  label: 'Voltar',
            events: { 
      		   click: () => DialogService.hide(),
      		  }
      		}
      	  ],
      	});
       } else {
         this.carregandoModal.closePersistentModal();
         Router.replace(this.origem);
       }
    });
  }
  
  public calculaMinuto(minutos: string) {
    let tempoTotal = (Math.floor(Number(minutos.replace(':', '')) / 100)) * 60;
    tempoTotal += Math.floor(Number(minutos.replace(':', '')) % 100);
    return tempoTotal;
  }
}
